////////////////////////////////////////////////////////////////////////////////////////

string grenade_mdl = <grenade.mdl>; // always define ent_created entities as strings!
string explosion_pcx = <explo+9.pcx>; // explosion sprite

////////////////////////////////////////////////////////////////////////////////////////

sound grenade_wav = <grenade.wav>; // grenade sound
sound explosion_wav = <explosion.wav>; // explosion sound

////////////////////////////////////////////////////////////////////////////////////////

function fire_grenade(); // creates the grenades
function move_grenade(); // moves the grenades
function grenade_explodes(); // controls the explosion sprite
function bounce_grenade(); // grenade's event function

////////////////////////////////////////////////////////////////////////////////////////

entity* weapon3; // pointer to our grenade launcher

////////////////////////////////////////////////////////////////////////////////////////

action grenade_launcher
{
	weapon3 = my; // I'm the 3rd weapon
	var player1_pos; // stores the initial position of the player
	var player2_pos; // stores the position of the player after a frame
	var weapon_height;
	var weapon_offset;
	my.passable = on; // the weapon model is passable
	while (player == null) {wait (1);} // wait until the player is created
	while (1)
	{
		vec_set (player1_pos.x, player.x); // store the initial player position
		// place the weapon 50 quants in front, 18 quants to the right and 20 quants below camera's origin
		vec_set (weapon_offset.x, vector (50, -18, -20));
		if (vec_dist (player1_pos.x, player2_pos.x) != 0) // the player has moved during the last frame?
		{
			weapon_height += 30 * time_step; // then offset weapon_height (30 = weapon waving speed)
			weapon_offset.z += 0.3 * sin(weapon_height); // (0.3 = weapon waving amplitude)
		}
		// rotate weapon_offset according to the camera angles
		vec_rotate (weapon_offset.x, vector (camera.pan, camera.tilt, 0)); 
		vec_add (weapon_offset.x, camera.x); // add the camera position to weapon_offset
		vec_set (my.x, weapon_offset.x); // set weapon's coords to weapon_offset
		my.pan = camera.pan; // use the same camera angles for the weapon
		my.tilt = camera.tilt;
		vec_set (player2_pos.x, player.x); // store the new player coordinates after 1 frame
		wait (1);
	}
}

on_mouse_left = fire_grenade; // call function fire_grenade() when the left mouse button is pressed

function fire_grenade()
{
	vec_for_vertex (temp.x, weapon3, 37); // get the coordinates for the grenade's origin
	 // create the grenade at camera's position and attach it the "move_grenade" function
	ent_create (grenade_mdl, temp.x, move_grenade);
	snd_play (grenade_wav, 100, 0); // play the grenade sound at a volume of 100
}

function move_grenade()
{
	my.skill99 = 1234; // unique identifier for a grenade entity, used inside dummies.wdl
	my.pan = camera.pan; // the grenade has the same pan and tilt with the camera (the player)
	my.tilt = camera.tilt;
	my.enable_impact = on; // the grenade is sensitive to impact with entities
	my.enable_entity = on;
	my.enable_block = on; // and with level blocks
	my.event = bounce_grenade; // its event function is triggered on collisions
	my.skill1 = 100 * time_step; // 100 gives the horizontal movement speed
	my.skill2 = 0; // no sideways movement
	my.skill3 = 10 * time_step; // vertical movement speed
	my.skill10 = 0; // not needed, but added for clarity
	while (my.skill10 < 3) // allow each bomb to live for 3 seconds
	{
		my.skill4 = c_move (my, my.skill1, nullvector, ignore_passable);
		my.skill3 -= 3 * time_step; // decrease the vertical speed constantly
		my.skill10 += time_step / 16; // increase my.skill10 every frame
		wait (1);
	}
	// the grenade has to explode here
	my.invisible = on; // so let's make it invisible and passable
	my.passable = on;
	ent_create (explosion_pcx, my.x, grenade_explodes); // create the explosion sprite
	snd_play (explosion_wav, 80, 0); // play the explosion sound
	my.skill10 = 0; // reset skill10
	while (my.skill10 < 2)
	{
		// scan around the grenade for 2 seconds, causing damage to the enemies nearby
		c_scan(my.x, my.pan, vector(360, 180, 200), ignore_me); // damage the enemies that are closer than 200 quants
		my.skill10 += time_step / 16;
		wait (1);
	}
	ent_remove (my); // remove the grenade model
}

function bounce_grenade()
{
	if (my.skill4 > 5) // the grenade is still moving with more than 5 quants per second?
	{
		vec_to_angle (my.pan, bounce); // then allow it to bounce!
	}
	my.skill1 *= 0.75; // decrease the horizontal movement speed on impact
	if (bounce.z != 0) // bounced vertically?
	{
		my.skill3 *= -0.15; // then decrease the vertical movement speed as well
		if (abs(my.skill3) < 0.01) // not much of a bounce here?
		{
			vec_set (my.skill1, nullvector); // then stop the grenade completely!
		}
	}
}

function grenade_explodes() // drives the explosion sprite
{
	my.passable = on; // the explosion sprite is passable
	my.z += 50; // move the sprite upwards a bit - looks better this way
	my.scale_x = 2; // we scale it up to 2
	my.scale_y = my.scale_x; // on both axis
	my.flare = on; // we set its flare
	my.bright = on; // and bright flags
	my.ambient = 100; // and then we give it an ambient of 100
	my.roll = random(360); // we set a random roll angle for it
	my.transparent = on; // and make it transparent
	my.alpha = 100; // but we set it to be completely opaque for now
	while (my.frame < 9) // go through all the animation frames
	{
		my.frame += 1 * time_step; // 1 gives the animation speed
      	wait (1);
	}
	while (my.alpha > 0) // now fade the last frame quickly
	{
		my.alpha -= 50 * time_step; // 50 = fading speed
		wait (1);
	}
	ent_remove (my); // time to remove the explosion sprite
}
